import axios from 'axios';
import { getConfig } from '../config/store.js';
const BASE_URL = 'https://api.clickup.com/api/v2';
const getAxiosInstance = async () => {
    const config = await getConfig();
    return axios.create({
        baseURL: BASE_URL,
        headers: {
            'Authorization': config.clickup.token,
            'Content-Type': 'application/json'
        }
    });
};
// Priority mapping based on ClickUp's API
export const PRIORITY_MAP = {
    1: { id: "1", priority: "urgent", color: "#f50000", orderindex: "1" },
    2: { id: "2", priority: "high", color: "#ffcc00", orderindex: "2" },
    3: { id: "3", priority: "normal", color: "#6fddff", orderindex: "3" },
    4: { id: "4", priority: "low", color: "#d8d8d8", orderindex: "4" }
};
// Add a delay utility function
const delay = (ms) => new Promise(resolve => setTimeout(resolve, ms));
// Add retry logic utility
async function retry(operation, retries = 3, delayMs = 1000) {
    try {
        return await operation();
    }
    catch (error) {
        if (retries === 0)
            throw error;
        await delay(delayMs);
        return retry(operation, retries - 1, delayMs);
    }
}
export async function createTask(listId, name, description, priority, status, dueDate, parent) {
    const axiosInstance = await getAxiosInstance();
    const taskData = {
        name,
        description,
        priority: priority || null,
        status: status || undefined
    };
    if (dueDate) {
        taskData.due_date = new Date(dueDate).getTime();
    }
    if (parent) {
        taskData.parent = parent;
    }
    console.log('Creating task with:', JSON.stringify(taskData, null, 2));
    console.log('List ID:', listId);
    const response = await axiosInstance.post(`/list/${listId}/task`, taskData);
    console.log('API Response:', JSON.stringify(response.data, null, 2));
    return response.data;
}
export async function createSubtask(parentId, name, description, priority, status) {
    const axiosInstance = await getAxiosInstance();
    const config = await getConfig();
    if (!config.clickup?.defaultList) {
        throw new Error('Default list not set. Please run "task config --interactive" first.');
    }
    const payload = {
        name,
        description,
        parent: parentId,
        priority: priority || undefined,
        status: status || undefined
    };
    // Add retry logic and delay for API consistency
    return retry(async () => {
        const response = await axiosInstance.post(`/list/${config.clickup.defaultList}/task`, payload);
        return response.data;
    });
}
export async function listTasks(listId) {
    const config = await getConfig();
    const targetListId = listId || config.clickup.defaultList;
    if (!targetListId) {
        throw new Error('No list ID specified. Please set a default list or provide a list ID.');
    }
    const axiosInstance = await getAxiosInstance();
    return retry(async () => {
        // Get all tasks including subtasks with expanded fields
        const query = new URLSearchParams({
            subtasks: 'true',
            include_closed: 'true',
            order_by: 'created',
            reverse: 'true',
            include_subtasks: 'true'
        }).toString();
        const response = await axiosInstance.get(`/list/${targetListId}/task?${query}`);
        // Create a map of tasks by ID for quick lookup
        const tasksById = new Map();
        // First pass: Initialize all tasks
        response.data.tasks.forEach(task => {
            tasksById.set(task.id, {
                ...task,
                subtasks: []
            });
        });
        // Second pass: Build task hierarchy
        response.data.tasks.forEach(task => {
            const parentId = typeof task.parent === 'string' ? task.parent : task.parent?.id;
            if (parentId) {
                const parentTask = tasksById.get(parentId);
                if (parentTask && parentTask.subtasks) {
                    parentTask.subtasks.push(tasksById.get(task.id));
                }
            }
        });
        // Return only root tasks (tasks without parents)
        return Array.from(tasksById.values()).filter(task => !task.parent);
    });
}
export async function getTask(taskId) {
    const axiosInstance = await getAxiosInstance();
    return retry(async () => {
        // Add query parameters to include subtasks
        const query = new URLSearchParams({
            include_subtasks: 'true',
            custom_fields: 'true'
        }).toString();
        const response = await axiosInstance.get(`/task/${taskId}?${query}`);
        return response.data;
    });
}
export async function updateTask(taskId, updates) {
    const axiosInstance = await getAxiosInstance();
    const requestBody = {
        ...updates,
        priority: updates.priority
    };
    const response = await axiosInstance.put(`/task/${taskId}`, requestBody);
    return response.data;
}
export async function deleteTask(taskId) {
    const axiosInstance = await getAxiosInstance();
    await axiosInstance.delete(`/task/${taskId}`);
}
export async function getListStatuses(listId) {
    const axiosInstance = await getAxiosInstance();
    const response = await axiosInstance.get(`/list/${listId}`);
    return response.data.statuses;
}
export async function getWorkspaces() {
    const axiosInstance = await getAxiosInstance();
    const response = await axiosInstance.get('/team');
    return response.data.teams;
}
export async function getSpaces(workspaceId) {
    const axiosInstance = await getAxiosInstance();
    const response = await axiosInstance.get(`/team/${workspaceId}/space`);
    return response.data.spaces;
}
export async function getLists(spaceId) {
    const axiosInstance = await getAxiosInstance();
    const response = await axiosInstance.get(`/space/${spaceId}/list`);
    return response.data.lists;
}
export async function listSubtasks(taskId) {
    const axiosInstance = await getAxiosInstance();
    const response = await axiosInstance.get(`/task/${taskId}/subtask`);
    return response.data.data.tasks;
}
export async function createStatus(listId, status, color, orderindex) {
    const axiosInstance = await getAxiosInstance();
    const payload = {
        status,
        color,
        orderindex: orderindex !== undefined ? orderindex.toString() : undefined
    };
    const response = await axiosInstance.post(`/list/${listId}/status`, payload);
    return response.data;
}
export async function updateStatus(listId, oldStatus, newStatus, color, orderindex) {
    const axiosInstance = await getAxiosInstance();
    const encodedOldStatus = encodeURIComponent(oldStatus);
    const response = await axiosInstance.put(`/list/${listId}/status/${encodedOldStatus}`, {
        status: newStatus,
        color,
        orderindex
    });
    return response.data.data;
}
export async function deleteStatus(listId, status) {
    const axiosInstance = await getAxiosInstance();
    const encodedStatus = encodeURIComponent(status);
    await axiosInstance.delete(`/list/${listId}/status/${encodedStatus}`);
}
export async function getSpaceTags(spaceId) {
    const axiosInstance = await getAxiosInstance();
    const response = await axiosInstance.get(`/space/${spaceId}/tag`);
    return response.data.tags;
}
export async function createTag(spaceId, name, tagBg = '#000000', tagFg = '#ffffff') {
    const axiosInstance = await getAxiosInstance();
    const tagData = {
        name,
        tag_bg: tagBg,
        tag_fg: tagFg
    };
    const response = await axiosInstance.post(`/space/${spaceId}/tag`, { tag: tagData });
    return response.data.tag;
}
export async function deleteTag(spaceId, name) {
    const axiosInstance = await getAxiosInstance();
    await axiosInstance.delete(`/space/${spaceId}/tag/${name}`);
}
export async function updateTaskTags(taskId, tags) {
    const axiosInstance = await getAxiosInstance();
    const response = await axiosInstance.put(`/task/${taskId}`, { tags });
    return response.data.data;
}
export async function manageStatus(listId, status, color, orderindex) {
    const axiosInstance = await getAxiosInstance();
    try {
        // First try to get existing statuses
        const existingStatuses = await getListStatuses(listId);
        const existingStatus = existingStatuses.find(s => s.status.toLowerCase() === status.toLowerCase());
        if (existingStatus) {
            // If status exists, update it
            const encodedStatus = encodeURIComponent(existingStatus.status);
            const response = await axiosInstance.put(`/list/${listId}/status/${encodedStatus}`, {
                status,
                color,
                orderindex: orderindex.toString()
            });
            return response.data.data;
        }
        else {
            // If status doesn't exist, create it
            const response = await axiosInstance.post(`/list/${listId}/status`, {
                status,
                color,
                orderindex: orderindex.toString()
            });
            return response.data;
        }
    }
    catch (error) {
        let message = 'Unknown error';
        if (error instanceof Error) {
            message = error.message;
        }
        // Type guard for axios error response
        if (error && typeof error === 'object' && 'response' in error) {
            const axiosError = error;
            if (axiosError.response?.data?.message) {
                message = axiosError.response.data.message;
            }
        }
        throw new Error(`Failed to manage status ${status}: ${message}`);
    }
}
